//-----------------------------------------------------------------------
// <copyright file="SubscriptionInfo.cs" company="iron9light">
//      Copyright (c) iron9light. All rights reserved.
// </copyright>
// <author>iron9light</author>
//-----------------------------------------------------------------------

namespace EventBrokerExtension
{
    using System;
    using System.Reflection;

    /// <summary>
    /// The subscription info.
    /// </summary>
    public struct SubscriptionInfo : IEquatable<SubscriptionInfo>
    {
        private readonly string eventTopic;
        private readonly MethodInfo method;

        /// <summary>
        /// Initializes a new instance of the <see cref="SubscriptionInfo"/> struct.
        /// </summary>
        /// <param name="eventTopic">Name of the event topic.</param>
        /// <param name="method">The method.</param>
        public SubscriptionInfo(string eventTopic, MethodInfo method)
        {
            this.eventTopic = eventTopic;
            this.method = method;
        }

        /// <summary>
        /// Gets the name of the event topic.
        /// </summary>
        /// <value>The name of the event topic.</value>
        public string EventTopic
        {
            get { return this.eventTopic; }
        }

        /// <summary>
        /// Gets the method.
        /// </summary>
        /// <value>The method.</value>
        public MethodInfo Method
        {
            get { return this.method; }
        }

        /// <summary>
        /// Implements the operator ==.
        /// </summary>
        /// <param name="left">The left.</param>
        /// <param name="right">The right.</param>
        /// <returns>The result of the operator.</returns>
        public static bool operator ==(SubscriptionInfo left, SubscriptionInfo right)
        {
            return left.Equals(right);
        }

        /// <summary>
        /// Implements the operator !=.
        /// </summary>
        /// <param name="left">The left.</param>
        /// <param name="right">The right.</param>
        /// <returns>The result of the operator.</returns>
        public static bool operator !=(SubscriptionInfo left, SubscriptionInfo right)
        {
            return !left.Equals(right);
        }

        /// <summary>
        /// Returns a hash code for this instance.
        /// </summary>
        /// <returns>
        /// A hash code for this instance, suitable for use in hashing algorithms and data structures like a hash table. 
        /// </returns>
        public override int GetHashCode()
        {
            return this.method.GetHashCode();
        }

        /// <summary>
        /// Determines whether the specified <see cref="System.Object"/> is equal to this instance.
        /// </summary>
        /// <param name="obj">The <see cref="System.Object"/> to compare with this instance.</param>
        /// <returns>
        ///     <c>true</c> if the specified <see cref="System.Object"/> is equal to this instance; otherwise, <c>false</c>.
        /// </returns>
        public override bool Equals(object obj)
        {
            if (!(obj is SubscriptionInfo))
            {
                return false;
            }

            return this.Equals((SubscriptionInfo)obj);
        }

        /// <summary>
        /// Indicates whether the current object is equal to another object of the same type.
        /// </summary>
        /// <param name="other">An object to compare with this object.</param>
        /// <returns>
        /// true if the current object is equal to the <paramref name="other"/> parameter; otherwise, false.
        /// </returns>
        public bool Equals(SubscriptionInfo other)
        {
            return this.method == other.method && this.eventTopic == other.eventTopic;
        }
    }
}