﻿//-----------------------------------------------------------------------
// <copyright file="ConfigurationFixtureBase.cs" company="iron9light">
//      Copyright (c) iron9light. All rights reserved.
// </copyright>
// <author>iron9light</author>
//-----------------------------------------------------------------------

namespace Tests.EventBrokerExtension.Configuration
{
    using System.Configuration;

    using EventBroker;

    using global::EventBrokerExtension;

    using Microsoft.Practices.Unity;
    using Microsoft.Practices.Unity.Configuration;

    public abstract class ConfigurationFixtureBase
    {
        private IUnityContainer container;

        protected abstract string ConfigName { get; }

        protected IUnityContainer Container
        {
            get
            {
                if (this.container == null)
                {
                    this.container = GetConfiguredContainer(this.ConfigName);
                }

                return this.container;
            }
        }

        protected IEventBroker Broker
        {
            get
            {
                IEventBrokerConfiguration configuration = this.Container.Configure<IEventBrokerConfiguration>();
                return configuration.EventBroker;
            }
        }

        protected T ResolveConfiguredObject<T>()
        {
            return this.Container.Resolve<T>();
        }

        protected UnityConfigurationSection GetUnitySection()
        {
            return GetUnitySection(this.ConfigName);
        }

        private static IUnityContainer GetConfiguredContainer(string baseName)
        {
            UnityConfigurationSection section = GetUnitySection(baseName);
            IUnityContainer container = new UnityContainer();
            section.Containers[0].Configure(container);
            return container;
        }

        private static UnityConfigurationSection GetUnitySection(string baseName)
        {
            Configuration config = OpenConfigFile(baseName);
            return (UnityConfigurationSection)config.GetSection("unity");
        }

        private static Configuration OpenConfigFile(string baseName)
        {
            ExeConfigurationFileMap map = new ExeConfigurationFileMap();
            map.ExeConfigFilename = baseName + ".config";
            return ConfigurationManager.OpenMappedExeConfiguration(map, ConfigurationUserLevel.None);
        }
    }
}
