﻿//-----------------------------------------------------------------------
// <copyright file="DefaultPublicationSelectorPolicyFixture.cs" company="iron9light">
//      Copyright (c) iron9light. All rights reserved.
// </copyright>
// <author>iron9light</author>
//-----------------------------------------------------------------------

namespace Tests.EventBrokerExtension
{
    using System.Collections.Generic;

    using global::EventBrokerExtension;

    using Microsoft.Practices.ObjectBuilder2;

    using Rhino.Mocks;

    using TestObjects;

    using Utility;

    using Xunit;

    public class DefaultPublicationSelectorPolicyFixture
    {
        [Fact]
        public void ReflectingOverObjectWithoutSubscriptionResultsInEmptyPolicy()
        {
            MockBuilderContext context = CreateContext();
            NamedTypeBuildKey buildKey = NamedTypeBuildKey.Make<object>();

            context.ExecuteBuildUp(buildKey, null);

            IPublicationSelectorPolicy publicationSelectorPolicy =
                    context.Policies.Get<IPublicationSelectorPolicy>(buildKey);

            Assert.NotNull(publicationSelectorPolicy);

            List<PublicationInfo> publicationInfos =
                    new List<PublicationInfo>(publicationSelectorPolicy.GetPublicationInfos(context));

            Assert.Equal(0, publicationInfos.Count);
        }

        [Fact]
        public void StrategyDoesntOverwriteAnExistingPolicy()
        {
            MockBuilderContext context = CreateContext();

            NamedTypeBuildKey buildKey = NamedTypeBuildKey.Make<object>();
            IPublicationSelectorPolicy policy = MockRepository.GenerateMock<IPublicationSelectorPolicy>();
            context.Policies.Set(policy, buildKey);

            context.ExecuteBuildUp(buildKey, null);

            IPublicationSelectorPolicy setPolicy =
                context.Policies.Get<IPublicationSelectorPolicy>(buildKey);
            Assert.Same(policy, setPolicy);
        }

        [Fact]
        public void ReflectingOverPublishingTypeResultsInCorrectPolicy()
        {
            MockBuilderContext context = CreateContext();
            NamedTypeBuildKey buildKey = NamedTypeBuildKey.Make<OneEventPublisher>();

            context.ExecuteBuildUp(buildKey, null);

            IPublicationSelectorPolicy policy = context.Policies.Get<IPublicationSelectorPolicy>(buildKey);

            Assert.NotNull(policy);

            List<PublicationInfo> publications = new List<PublicationInfo>(policy.GetPublicationInfos(context));

            CollectionAssert.AreEqual(new PublicationInfo[] { new PublicationInfo("paste", typeof(OneEventPublisher).GetEvent("Pasting")) }, publications);
        }

        [Fact]
        public void ReflectingOverSubscribingTypeResultsInCorrectPolicy()
        {
            MockBuilderContext context = CreateContext();
            NamedTypeBuildKey buildKey = NamedTypeBuildKey.Make<OneEventSubscriber>();

            context.ExecuteBuildUp(buildKey, null);

            IPublicationSelectorPolicy policy = context.Policies.Get<IPublicationSelectorPolicy>(buildKey);

            Assert.NotNull(policy);

            List<PublicationInfo> publications = new List<PublicationInfo>(policy.GetPublicationInfos(context));

            Assert.Equal(0, publications.Count);
        }

        [Fact]
        public void OneTypeCanPublishAndSubscribeMultipleTimes()
        {
            MockBuilderContext context = CreateContext();
            NamedTypeBuildKey buildKey = NamedTypeBuildKey.Make<ClipboardManager>();

            context.ExecuteBuildUp(buildKey, null);

            IPublicationSelectorPolicy policy = context.Policies.Get<IPublicationSelectorPolicy>(buildKey);

            Assert.NotNull(policy);

            List<PublicationInfo> publications = new List<PublicationInfo>(policy.GetPublicationInfos(context));

            publications.Sort(
                delegate(PublicationInfo a, PublicationInfo b)
                {
                    return a.EventTopic.CompareTo(b.EventTopic);
                });

            CollectionAssert.AreEqual(
                new PublicationInfo[]
                {
                    new PublicationInfo("copy", typeof(ClipboardManager).GetEvent("Copy")),
                    new PublicationInfo("cut", typeof(ClipboardManager).GetEvent("Cut")),
                    new PublicationInfo("paste", typeof(ClipboardManager).GetEvent("Paste")),
                },
                publications);
        }

        private static MockBuilderContext CreateContext()
        {
            MockBuilderContext context = new MockBuilderContext();
            context.Policies.SetDefault<IPublicationSelectorPolicy>(new DefaultPublicationSelectorPolicy());

            return context;
        }
    }
}
