//-----------------------------------------------------------------------
// <copyright file="EventBrokerExtensionFixture.cs" company="iron9light">
//      Copyright (c) iron9light. All rights reserved.
// </copyright>
// <author>iron9light</author>
//-----------------------------------------------------------------------

namespace Tests.EventBrokerExtension
{
    using System;

    using EventBroker;

    using global::EventBrokerExtension;

    using Microsoft.Practices.ObjectBuilder2;
    using Microsoft.Practices.Unity;

    using Rhino.Mocks;

    using TestObjects;

    using Xunit;

    public class EventBrokerExtensionFixture
    {
        [Fact]
        public void TestIEventBrokerConfiguration()
        {
            IUnityContainer container = new UnityContainer().AddNewExtension<EventBrokerExtension>();
            IEventBrokerConfiguration configuration = container.Configure<IEventBrokerConfiguration>();
            Assert.NotNull(configuration);
            Assert.NotNull(configuration.EventBroker);
        }

        [Fact]
        public void HaveBrokerInLocatorAndTwoPolicies()
        {
            UnityContainer container = new UnityContainer();
            container.AddNewExtension<EventBrokerExtension>();
            container.AddNewExtension<MockExtension>();
        }

        [Fact]
        public void TestWireupStrategies()
        {
            UnityContainer container = new UnityContainer();
            container.AddNewExtension<EventBrokerExtension>();

            MockRepository mocks = new MockRepository();
            IEventBroker broker = mocks.StrictMock<IEventBroker>();

            container.AddExtension(new MockExtensionForMockBroker(broker));

            ClipboardManager existing = new ClipboardManager();

            using (mocks.Unordered())
            {
                broker.RegisterPublication("cut", existing, "Cut");
                broker.RegisterPublication("copy", existing, "Copy");
                broker.RegisterPublication("paste", existing, "Paste");

                broker.RegisterSubscription("copy", existing, "OnCopy", new Type[] { typeof(object), typeof(EventArgs) });
                broker.RegisterSubscription(
                        "clipboard data available",
                        existing,
                        "OnClipboardDataAvailable",
                        new Type[] { typeof(object), typeof(EventArgs) });
            }

            mocks.ReplayAll();

            container.BuildUp(existing);

            mocks.VerifyAll();
        }

        private class MockExtension : UnityContainerExtension
        {
            protected override void Initialize()
            {
                IEventBroker broker = this.Context.Locator.Get<IEventBroker>(typeof(IEventBroker));
                Assert.NotNull(broker);

                IBuildKey buildKey = new NamedTypeBuildKey(typeof(object));
                IPublicationSelectorPolicy publicationSelectorPolicy = this.Context.Policies.Get<IPublicationSelectorPolicy>(buildKey);
                Assert.NotNull(publicationSelectorPolicy);

                ISubscriptionSelectorPolicy subscriptionSelectorPolicy =
                        this.Context.Policies.Get<ISubscriptionSelectorPolicy>(buildKey);
                Assert.NotNull(subscriptionSelectorPolicy);
            }
        }

        private class MockExtensionForMockBroker : UnityContainerExtension
        {
            private readonly IEventBroker broker;

            public MockExtensionForMockBroker(IEventBroker broker)
            {
                this.broker = broker;
            }

            protected override void Initialize()
            {
                this.Context.Locator.Remove(typeof(IEventBroker));
                this.Context.Locator.Add(typeof(IEventBroker), this.broker);

                Assert.Same(this.broker, this.Context.Locator.Get(typeof(IEventBroker)));
            }
        }
    }
}