﻿//-----------------------------------------------------------------------
// <copyright file="EventBrokerStrategyBaseFixture.cs" company="iron9light">
//      Copyright (c) iron9light. All rights reserved.
// </copyright>
// <author>iron9light</author>
//-----------------------------------------------------------------------

namespace Tests.EventBrokerExtension
{
    using System;

    using EventBroker;

    using global::EventBrokerExtension;

    using Microsoft.Practices.ObjectBuilder2;

    using Rhino.Mocks;

    using Utility;

    using Xunit;

    public class EventBrokerStrategyBaseFixture
    {
        [Fact]
        public void NoEventBroker()
        {
            MockBuilderContext context = new MockBuilderContext();

            MockEventBrokerStrategy strategy = new MockEventBrokerStrategy();

            Assert.Throws<InvalidOperationException>(
                    delegate
                    {
                        strategy.GetEventBroker(context);
                    });
        }

        [Fact]
        public void ExistEventBroker()
        {
            MockBuilderContext context = new MockBuilderContext();

            IEventBroker eventBroker = MockRepository.GenerateMock<IEventBroker>();

            context.Locator.Add(typeof(IEventBroker), eventBroker);

            MockEventBrokerStrategy strategy = new MockEventBrokerStrategy();

            IEventBroker returnedBroker = strategy.GetEventBroker(context);

            Assert.Same(eventBroker, returnedBroker);
        }

        private class MockEventBrokerStrategy : EventBrokerStrategyBase
        {
            public IEventBroker GetEventBroker(IBuilderContext context)
            {
                return GetBroker(context);
            }
        }
    }
}
