﻿//-----------------------------------------------------------------------
// <copyright file="ParameterMatcherFixture.cs" company="iron9light">
//      Copyright (c) iron9light. All rights reserved.
// </copyright>
// <author>iron9light</author>
//-----------------------------------------------------------------------

namespace Tests.EventBrokerExtension
{
    using System;
    using System.Collections.Generic;

    using global::EventBrokerExtension;

    using Xunit;

    public class ParameterMatcherFixture
    {
        [Fact]
        public void EmptyParameterListMatches()
        {
            ParameterMatcher matcher = new ParameterMatcher(Parameters());

            Assert.True(matcher.Matches(Types()));
        }

        [Fact]
        public void MismatchedParameterListsDontMatch()
        {
            ParameterMatcher matcher = new ParameterMatcher(Parameters());
            Assert.False(matcher.Matches(Types(typeof(int))));
        }

        [Fact]
        public void SameLengthDifferentTypesDontMatch()
        {
            ParameterMatcher matcher = new ParameterMatcher(Parameters(typeof(int)));
            Assert.False(matcher.Matches(Types(typeof(string))));
        }

        [Fact]
        public void SameLengthSameTypesMatch()
        {
            ParameterMatcher matcher = new ParameterMatcher(Parameters(typeof(int), typeof(string)));
            Assert.True(matcher.Matches(Types(typeof(int), typeof(string))));
        }

        [Fact]
        public void OpenGenericTypesMatch()
        {
            ParameterMatcher matcher = new ParameterMatcher(Parameters(typeof(IList<>), typeof(IList<>)));
            Assert.True(matcher.Matches(Types(typeof(IList<>), typeof(IList<>))));
        }

        private static MethodParameter[] Parameters(params Type[] types)
        {
            List<MethodParameter> values = new List<MethodParameter>(MethodParameter.ToParameters(types));
            return values.ToArray();
        }

        private static Type[] Types(params Type[] types)
        {
            return types;
        }
    }
}