//-----------------------------------------------------------------------
// <copyright file="RegistrationPublicationFixture.cs" company="iron9light">
//      Copyright (c) iron9light. All rights reserved.
// </copyright>
// <author>iron9light</author>
//-----------------------------------------------------------------------

namespace Tests.EventBrokerExtension
{
    using System;
    using System.Collections.Generic;

    using global::EventBrokerExtension;

    using Microsoft.Practices.ObjectBuilder2;
    using Microsoft.Practices.Unity;

    using TestObjects;

    using Utility;

    using Xunit;

    public class RegistrationPublicationFixture : RegistrationMemberFixtureBase
    {
        [Fact]
        public void QualifyingInjectionPublicationCanBeConfigured()
        {
            IUnityContainer container =
                    this.CreateContainer()
                    .Configure<InjectedMembers>()
                    .ConfigureInjectionFor<ClipboardManager>(new RegistrationPublication("Cut", "cut me"))
                    .Container;

            ClipboardManager existing = container.Resolve<ClipboardManager>();

            IBuildKey buildKey = NamedTypeBuildKey.Make<ClipboardManager>();
            IPublicationSelectorPolicy policy =
                    this.Policies.Get<IPublicationSelectorPolicy>(buildKey);

            Assert.IsType<SpecifiedPublicationSelectorPolicy>(policy);

            MockBuilderContext context = new MockBuilderContext();
            context.ExecuteBuildUp(buildKey, existing);
            IEnumerable<PublicationInfo> publicationInfos = policy.GetPublicationInfos(context);

            Assert.Contains(new PublicationInfo("cut me", typeof(ClipboardManager).GetEvent("Cut")), publicationInfos);
            Assert.DoesNotContain(new PublicationInfo("cut", typeof(ClipboardManager).GetEvent("Cut")), publicationInfos);
            Assert.DoesNotContain(new PublicationInfo("copy", typeof(ClipboardManager).GetEvent("Copy")), publicationInfos);
            Assert.DoesNotContain(new PublicationInfo("paste", typeof(ClipboardManager).GetEvent("Paste")), publicationInfos);
        }

        [Fact]
        public void CannotConfigureWrongEvent()
        {
            Assert.Throws<InvalidOperationException>(
                    delegate
                    {
                        this.CreateContainer()
                            .Configure<InjectedMembers>()
                            .ConfigureInjectionFor<ClipboardManager>(new RegistrationPublication("Fly", "fly"));
                    });
        }

        [Fact]
        public void CannotConfigureEventWithWrongParams()
        {
            Assert.Throws<InvalidOperationException>(
                    delegate
                    {
                        this.CreateContainer()
                            .Configure<InjectedMembers>()
                            .ConfigureInjectionFor<ObjectWithWrongParams>(new RegistrationPublication("EventWithOneParam", "one"));
                    });

            Assert.Throws<InvalidOperationException>(
                    delegate
                    {
                        this.CreateContainer()
                            .Configure<InjectedMembers>()
                            .ConfigureInjectionFor<ObjectWithWrongParams>(new RegistrationPublication("EventWithThreeParams", "three"));
                    });
        }

        [Fact]
        public void CannotConfigureEventWithReturn()
        {
            Assert.Throws<InvalidOperationException>(
                    delegate
                    {
                        this.CreateContainer()
                            .Configure<InjectedMembers>()
                            .ConfigureInjectionFor<ObjectWithReturn>(new RegistrationPublication("EventWithReturn", "return"));
                    });
        }

        [Fact]
        public void CannotConfigureStaticEvent()
        {
            Assert.Throws<InvalidOperationException>(
                    delegate
                    {
                        this.CreateContainer()
                            .Configure<InjectedMembers>()
                            .ConfigureInjectionFor<StaticObject>(new RegistrationPublication("StaticEvent", "static"));
                    });
        }

        [Fact]
        public void CannotConfigureEventWithOutParams()
        {
            Assert.Throws<InvalidOperationException>(
                    delegate
                    {
                        this.CreateContainer()
                            .Configure<InjectedMembers>()
                            .ConfigureInjectionFor<ObjectWithOutParams>(new RegistrationPublication("EventWithOutParams", "out"));
                    });
        }

        [Fact]
        public void CannotConfigureEventWithRefParams()
        {
            Assert.Throws<InvalidOperationException>(
                    delegate
                    {
                        this.CreateContainer()
                            .Configure<InjectedMembers>()
                            .ConfigureInjectionFor<ObjectWithRefParams>(new RegistrationPublication("EventWithRefParams", "ref"));
                    });
        }
    }
}