//-----------------------------------------------------------------------
// <copyright file="SpecifiedPublicationSelectorPolicyFixture.cs" company="iron9light">
//      Copyright (c) iron9light. All rights reserved.
// </copyright>
// <author>iron9light</author>
//-----------------------------------------------------------------------

namespace Tests.EventBrokerExtension
{
    using System;
    using System.Collections.Generic;

    using global::EventBrokerExtension;

    using Microsoft.Practices.ObjectBuilder2;

    using TestObjects;

    using Utility;

    using Xunit;

    public class SpecifiedPublicationSelectorPolicyFixture
    {
        [Fact]
        public void NoPublication()
        {
            SpecifiedPublicationSelectorPolicy policy = new SpecifiedPublicationSelectorPolicy();

            MockBuilderContext context = CreateContext<ClipboardManager>();

            IEnumerable<PublicationInfo> publicationInfos = policy.GetPublicationInfos(context);

            Assert.Empty(publicationInfos);
        }

        [Fact]
        public void HavePublication()
        {
            SpecifiedPublicationSelectorPolicy policy = new SpecifiedPublicationSelectorPolicy();
            policy.Add(typeof(ClipboardManager).GetEvent("Cut"), "cut");
            policy.Add(typeof(ClipboardManager).GetEvent("Copy"), "copy");

            MockBuilderContext context = CreateContext<ClipboardManager>();

            IEnumerable<PublicationInfo> publicationInfos = policy.GetPublicationInfos(context);

            Assert.Contains(new PublicationInfo("cut", typeof(ClipboardManager).GetEvent("Cut")), publicationInfos);
            Assert.Contains(new PublicationInfo("copy", typeof(ClipboardManager).GetEvent("Copy")), publicationInfos);
            Assert.DoesNotContain(new PublicationInfo("paste", typeof(ClipboardManager).GetEvent("Paste")), publicationInfos);
        }

        [Fact]
        public void GenericPublisher()
        {
            SpecifiedPublicationSelectorPolicy policy = new SpecifiedPublicationSelectorPolicy();
            policy.Add(typeof(GenericObject<>).GetEvent("Run"), "run");

            MockBuilderContext context = CreateContext<GenericObject<AssemblyLoadEventArgs>>();

            IEnumerable<PublicationInfo> publicationInfos = policy.GetPublicationInfos(context);

            Assert.Contains(new PublicationInfo("run", typeof(GenericObject<AssemblyLoadEventArgs>).GetEvent("Run")), publicationInfos);
            Assert.DoesNotContain(new PublicationInfo("run", typeof(GenericObject<>).GetEvent("Run")), publicationInfos);
        }

        private static MockBuilderContext CreateContext<T>()
            where T : new()
        {
            MockBuilderContext context = new MockBuilderContext();
            IBuildKey buildKey = new NamedTypeBuildKey(typeof(T));
            context.ExecuteBuildUp(buildKey, new T());
            return context;
        }
    }
}