//-----------------------------------------------------------------------
// <copyright file="SpecifiedSubscriptionSelectorPolicyFixture.cs" company="iron9light">
//      Copyright (c) iron9light. All rights reserved.
// </copyright>
// <author>iron9light</author>
//-----------------------------------------------------------------------

namespace Tests.EventBrokerExtension
{
    using System;
    using System.Collections.Generic;

    using global::EventBrokerExtension;

    using Microsoft.Practices.ObjectBuilder2;

    using TestObjects;

    using Utility;

    using Xunit;

    public class SpecifiedSubscriptionSelectorPolicyFixture
    {
        [Fact]
        public void NoSubscription()
        {
            SpecifiedSubscriptionSelectorPolicy policy = new SpecifiedSubscriptionSelectorPolicy();

            MockBuilderContext context = CreateContext<ClipboardManager>();

            IEnumerable<SubscriptionInfo> subscriptionInfos = policy.GetSubscriptionInfos(context);

            Assert.Empty(subscriptionInfos);
        }

        [Fact]
        public void HaveSubscription()
        {
            SpecifiedSubscriptionSelectorPolicy policy = new SpecifiedSubscriptionSelectorPolicy();
            policy.Add(typeof(ClipboardManager).GetMethod("OnCopy"), "copy");

            MockBuilderContext context = CreateContext<ClipboardManager>();

            IEnumerable<SubscriptionInfo> subscriptionInfos = policy.GetSubscriptionInfos(context);

            Assert.Contains(new SubscriptionInfo("copy", typeof(ClipboardManager).GetMethod("OnCopy")), subscriptionInfos);
            Assert.DoesNotContain(new SubscriptionInfo("clipboard data available", typeof(ClipboardManager).GetMethod("OnClipboardDataAvailable")), subscriptionInfos);
        }

        [Fact]
        public void GenericSubscriber()
        {
            SpecifiedSubscriptionSelectorPolicy policy = new SpecifiedSubscriptionSelectorPolicy();
            policy.Add(typeof(GenericObject<>).GetMethod("OnRun"), "run");

            MockBuilderContext context = CreateContext<GenericObject<ConsoleCancelEventArgs>>();

            IEnumerable<SubscriptionInfo> subscriptionInfos = policy.GetSubscriptionInfos(context);

            Assert.Contains(new SubscriptionInfo("run", typeof(GenericObject<ConsoleCancelEventArgs>).GetMethod("OnRun")), subscriptionInfos);
            Assert.DoesNotContain(new SubscriptionInfo("run", typeof(GenericObject<>).GetMethod("OnRun")), subscriptionInfos);
        }

        private static MockBuilderContext CreateContext<T>()
            where T : new()
        {
            MockBuilderContext context = new MockBuilderContext();
            IBuildKey buildKey = new NamedTypeBuildKey(typeof(T));
            context.ExecuteBuildUp(buildKey, new T());
            return context;
        }
    }
}