using System;
using System.Collections.Generic;
using System.Reflection;
using Unity.Extensions;
using Xunit;

namespace Unity.Extensions
{
    public class GenericMethodsInterfaceInterceptorTest
    {
        [Fact]
        public void GenericMethod()
        {
            Recorder.Records.Clear();
            RecordingHandler handler = new RecordingHandler();
            MethodBase method = typeof(INonGenericSpy).GetMethod("GenericMethod");
            Dictionary<MethodBase, List<IInterceptionHandler>> dictionary = new Dictionary<MethodBase, List<IInterceptionHandler>>();
            List<IInterceptionHandler> handlers = new List<IInterceptionHandler> { handler };
            dictionary.Add(method, handlers);

            INonGenericSpy result = WrapAndCreateType<INonGenericSpy, NonGenericSpy>(dictionary);
            result.GenericMethod(21);

            Assert.Equal(3, Recorder.Records.Count);
            Assert.Equal("Before Method", Recorder.Records[0]);
            Assert.Equal("In method with data 21", Recorder.Records[1]);
            Assert.Equal("After Method", Recorder.Records[2]);
        }

        [Fact]
        public void ReturnsDataOfGenericType()
        {
            Recorder.Records.Clear();
            RecordingHandler handler = new RecordingHandler();
            MethodBase method = typeof(INonGenericSpy).GetMethod("GenericReturn");
            Dictionary<MethodBase, List<IInterceptionHandler>> dictionary = new Dictionary<MethodBase, List<IInterceptionHandler>>();
            List<IInterceptionHandler> handlers = new List<IInterceptionHandler> { handler };
            dictionary.Add(method, handlers);

            INonGenericSpy result = WrapAndCreateType<INonGenericSpy, NonGenericSpy>(dictionary);
            int value = result.GenericReturn<int, double>(256.9);

            Assert.Equal(3, Recorder.Records.Count);
            Assert.Equal("Before Method", Recorder.Records[0]);
            Assert.Equal("In method with data 256.9", Recorder.Records[1]);
            Assert.Equal("After Method", Recorder.Records[2]);
            Assert.Equal(default(int), value);
        }

        [Fact]
        public void WhereClauseOnMethod()
        {
            Recorder.Records.Clear();
            RecordingHandler handler = new RecordingHandler();
            MethodBase method = typeof(INonGenericSpy).GetMethod("WhereMethod");
            Dictionary<MethodBase, List<IInterceptionHandler>> dictionary = new Dictionary<MethodBase, List<IInterceptionHandler>>();
            List<IInterceptionHandler> handlers = new List<IInterceptionHandler> { handler };
            dictionary.Add(method, handlers);

            INonGenericSpy result = WrapAndCreateType<INonGenericSpy, NonGenericSpy>(dictionary);
            result.WhereMethod(new Foo());

            Assert.Equal(3, Recorder.Records.Count);
            Assert.Equal("Before Method", Recorder.Records[0]);
            Assert.Equal("In method with data " + typeof(Foo).FullName, Recorder.Records[1]);
            Assert.Equal("After Method", Recorder.Records[2]);
        }

        static TInterface WrapAndCreateType<TInterface, TConcrete>(IEnumerable<KeyValuePair<MethodBase, List<IInterceptionHandler>>> handlers)
            where TConcrete : TInterface
        {
            Type wrappedType = InterfaceInterceptor.WrapInterface(typeof(TInterface));
            ILEmitProxy proxy = new ILEmitProxy(handlers);
            object target = Activator.CreateInstance(typeof(TConcrete));
            List<object> wrappedCtorArgs = new List<object> { proxy, target };
            return (TInterface)Activator.CreateInstance(wrappedType, wrappedCtorArgs.ToArray());
        }

        public class Foo : IFoo {}

        public interface IFoo {}

        public interface INonGenericSpy
        {
            void GenericMethod<T>(T data);
            T GenericReturn<T, T1>(T1 data);

            void WhereMethod<T>(T data)
                where T : class, IFoo;
        }

        public class NonGenericSpy : INonGenericSpy
        {
            public void GenericMethod<T>(T data)
            {
                Recorder.Records.Add("In method with data " + data);
            }

            public T GenericReturn<T, T1>(T1 data)
            {
                Recorder.Records.Add("In method with data " + data);
                return default(T);
            }

            public void WhereMethod<T>(T data)
                where T : class, IFoo
            {
                Recorder.Records.Add("In method with data " + data);
            }
        }
    }
}