using System;
using System.Configuration;
using Microsoft.Practices.Unity;
using Microsoft.Practices.Unity.Configuration;
using Xunit;

namespace Unity.Extensions
{
    public class InterceptInterfaceConfigurationTest
    {
        [Fact]
        public void InterceptConfiguration()
        {
            Recorder.Records.Clear();
            IUnityContainer container = new UnityContainer();
            UnityConfigurationSection section = (UnityConfigurationSection)ConfigurationManager.GetSection("unity");
            section.Containers.Default.Configure(container);
            container.RegisterType<ISpy, SpyInterface>();

            ISpy obj = container.Resolve<ISpy>();
            obj.InterceptedMethod();

            Assert.Equal(3, Recorder.Records.Count);
            Assert.Equal("Before Method", Recorder.Records[0]);
            Assert.Equal("In Method", Recorder.Records[1]);
            Assert.Equal("After Method", Recorder.Records[2]);
        }

        public interface ISpy
        {
            void InterceptedMethod();
            void ThrowsException();
        }

        internal sealed class SpyInterface : ISpy
        {
            public void InterceptedMethod()
            {
                Recorder.Records.Add("In Method");
            }

            public void ThrowsException()
            {
                Recorder.Records.Add("In Method");
                throw new Exception();
            }
        }
    }
}