using Microsoft.Practices.Unity;
using Unity.Extensions;
using Xunit;

namespace Unity.Extensions
{
    public class InterceptInterfacePropertiesInterceptionExtensionTest
    {
        [Fact]
        public void CanInterceptPropertySettersAndGetters()
        {
            Recorder.Records.Clear();
            IUnityContainer container = new UnityContainer();
            container.AddNewExtension<InterceptionExtension>();
            container.RegisterType<IFoo, Foo>();

            IFoo obj = container.Resolve<IFoo>();
            obj.MyProperty = "foo";
            string unused = obj.MyProperty;

            Assert.Equal(6, Recorder.Records.Count);
            Assert.Equal("Before Method", Recorder.Records[0]);
            Assert.Equal("PropSet: foo", Recorder.Records[1]);
            Assert.Equal("After Method", Recorder.Records[2]);
            Assert.Equal("Before Method", Recorder.Records[3]);
            Assert.Equal("PropGet", Recorder.Records[4]);
            Assert.Equal("After Method", Recorder.Records[5]);
        }

        public class Foo : IFoo
        {
            public string MyProperty
            {
                [InterfaceIntercept(typeof(RecordingHandler))]
                get
                {
                    Recorder.Records.Add("PropGet");
                    return null;
                }

                [InterfaceIntercept(typeof(RecordingHandler))]
                set { Recorder.Records.Add("PropSet: " + value); }
            }
        }

        public interface IFoo
        {
            string MyProperty { get; set; }
        }
    }
}