using Microsoft.Practices.Unity;
using Unity.Extensions;
using Xunit;

namespace Unity.Extensions
{
    public class InterceptVirtualGenericsInterceptionExtensionTest
    {
        [Fact]
        public void GenericMethodOnGenericClass()
        {
            Recorder.Records.Clear();
            IUnityContainer container = new UnityContainer();
            container.AddNewExtension<InterceptionExtension>();

            GenericFoo<int> obj = container.Resolve<GenericFoo<int>>();
            obj.Baz(21, 42);
            obj.Baz(96, "world");

            Assert.Equal(6, Recorder.Records.Count);
            Assert.Equal("Before Method", Recorder.Records[0]);
            Assert.Equal("Passed: 21, 42", Recorder.Records[1]);
            Assert.Equal("After Method", Recorder.Records[2]);
            Assert.Equal("Before Method", Recorder.Records[3]);
            Assert.Equal("Passed: 96, world", Recorder.Records[4]);
            Assert.Equal("After Method", Recorder.Records[5]);
        }

        [Fact]
        public void GenericMethodOnNonGenericClass()
        {
            Recorder.Records.Clear();
            IUnityContainer container = new UnityContainer();
            container.AddNewExtension<InterceptionExtension>();

            Foo obj = container.Resolve<Foo>();
            obj.Bar(42);
            obj.Bar("world");

            Assert.Equal(6, Recorder.Records.Count);
            Assert.Equal("Before Method", Recorder.Records[0]);
            Assert.Equal("Passed: 42", Recorder.Records[1]);
            Assert.Equal("After Method", Recorder.Records[2]);
            Assert.Equal("Before Method", Recorder.Records[3]);
            Assert.Equal("Passed: world", Recorder.Records[4]);
            Assert.Equal("After Method", Recorder.Records[5]);
        }

        [Fact]
        public void NonGenericMethodOnGenericClass()
        {
            Recorder.Records.Clear();
            IUnityContainer container = new UnityContainer();
            container.AddNewExtension<InterceptionExtension>();

            GenericFoo<int> obj = container.Resolve<GenericFoo<int>>();
            obj.Bar(42);

            Assert.Equal(3, Recorder.Records.Count);
            Assert.Equal("Before Method", Recorder.Records[0]);
            Assert.Equal("Passed: 42", Recorder.Records[1]);
            Assert.Equal("After Method", Recorder.Records[2]);
        }

        public class ATest : ITest
        {
            public string strtest = "Hello";
        }

        public class Foo
        {
            [VirtualIntercept(typeof(RecordingHandler))]
            public virtual void Bar<T>(T value)
            {
                Recorder.Records.Add("Passed: " + value);
            }
        }

        public class GenericFoo<T>
        {
            [VirtualIntercept(typeof(RecordingHandler))]
            public virtual void Bar(T value)
            {
                Recorder.Records.Add("Passed: " + value);
            }

            [VirtualIntercept(typeof(RecordingHandler))]
            public virtual void Baz<T1>(T value,
                                        T1 value2)
            {
                Recorder.Records.Add("Passed: " + value + ", " + value2);
            }
        }

        public interface ITest {}
    }
}