using System;
using System.Collections.Generic;
using System.Reflection;
using Unity.Extensions;
using Xunit;

namespace Unity.Extensions
{
    public class OutParametersInterfaceInterceptorTest
    {
        [Fact]
        public void OutComplexValueType()
        {
            RecordingHandler handler = new RecordingHandler();
            MethodBase method = typeof(ISpyOut).GetMethod("OutComplexValueType");
            Dictionary<MethodBase, List<IInterceptionHandler>> dictionary = new Dictionary<MethodBase, List<IInterceptionHandler>>();
            List<IInterceptionHandler> handlers = new List<IInterceptionHandler> { handler };
            dictionary.Add(method, handlers);

            ISpyOut result = WrapAndCreateType<ISpyOut, SpyOut>(dictionary);
            ComplexValueType outValue;
            result.OutComplexValueType(out outValue);

            Assert.Equal(byte.MaxValue, outValue.Byte);
            Assert.Equal('a', outValue.Char);
            Assert.Equal(decimal.MaxValue, outValue.Decimal);
            Assert.Equal(double.MaxValue, outValue.Double);
            Assert.Equal(float.MaxValue, outValue.Float);
            Assert.Equal(int.MaxValue, outValue.Int);
            Assert.Equal(long.MaxValue, outValue.Long);
            Assert.Equal(short.MaxValue, outValue.Short);
            Assert.Equal("Hello, world!", outValue.String);
            Assert.Equal(uint.MaxValue, outValue.UInt);
            Assert.Equal(ulong.MaxValue, outValue.ULong);
            Assert.Equal(ushort.MaxValue, outValue.UShort);
        }

        [Fact]
        public void OutDouble()
        {
            RecordingHandler handler = new RecordingHandler();
            MethodBase method = typeof(ISpyOut).GetMethod("OutDouble");
            Dictionary<MethodBase, List<IInterceptionHandler>> dictionary = new Dictionary<MethodBase, List<IInterceptionHandler>>();
            List<IInterceptionHandler> handlers = new List<IInterceptionHandler> { handler };
            dictionary.Add(method, handlers);

            ISpyOut result = WrapAndCreateType<ISpyOut, SpyOut>(dictionary);
            double outValue;
            result.OutDouble(out outValue);

            Assert.Equal(double.MaxValue, outValue);
        }

        [Fact]
        public void OutInt16()
        {
            RecordingHandler handler = new RecordingHandler();
            MethodBase method = typeof(ISpyOut).GetMethod("OutInt16");
            Dictionary<MethodBase, List<IInterceptionHandler>> dictionary = new Dictionary<MethodBase, List<IInterceptionHandler>>();
            List<IInterceptionHandler> handlers = new List<IInterceptionHandler> { handler };
            dictionary.Add(method, handlers);

            ISpyOut result = WrapAndCreateType<ISpyOut, SpyOut>(dictionary);
            short outValue;
            result.OutInt16(out outValue);

            Assert.Equal(short.MaxValue, outValue);
        }

        [Fact]
        public void OutInt32()
        {
            RecordingHandler handler = new RecordingHandler();
            MethodBase method = typeof(ISpyOut).GetMethod("OutInt32");
            Dictionary<MethodBase, List<IInterceptionHandler>> dictionary = new Dictionary<MethodBase, List<IInterceptionHandler>>();
            List<IInterceptionHandler> handlers = new List<IInterceptionHandler> { handler };
            dictionary.Add(method, handlers);

            ISpyOut result = WrapAndCreateType<ISpyOut, SpyOut>(dictionary);
            int outValue;
            result.OutInt32(out outValue);

            Assert.Equal(int.MaxValue, outValue);
        }

        [Fact]
        public void OutInt64()
        {
            RecordingHandler handler = new RecordingHandler();
            MethodBase method = typeof(ISpyOut).GetMethod("OutInt64");
            Dictionary<MethodBase, List<IInterceptionHandler>> dictionary = new Dictionary<MethodBase, List<IInterceptionHandler>>();
            List<IInterceptionHandler> handlers = new List<IInterceptionHandler> { handler };
            dictionary.Add(method, handlers);

            ISpyOut result = WrapAndCreateType<ISpyOut, SpyOut>(dictionary);
            long outValue;
            result.OutInt64(out outValue);

            Assert.Equal(long.MaxValue, outValue);
        }

        [Fact]
        public void OutReferenceType()
        {
            RecordingHandler handler = new RecordingHandler();
            MethodBase method = typeof(ISpyOut).GetMethod("OutReferenceType");
            Dictionary<MethodBase, List<IInterceptionHandler>> dictionary = new Dictionary<MethodBase, List<IInterceptionHandler>>();
            List<IInterceptionHandler> handlers = new List<IInterceptionHandler> { handler };
            dictionary.Add(method, handlers);

            ISpyOut result = WrapAndCreateType<ISpyOut, SpyOut>(dictionary);
            string outReference;
            result.OutReferenceType(out outReference);

            Assert.Equal("Hello, world!", outReference);
        }

        static TInterface WrapAndCreateType<TInterface, TConcrete>(IEnumerable<KeyValuePair<MethodBase, List<IInterceptionHandler>>> handlers)
            where TConcrete : TInterface
        {
            Type wrappedType = InterfaceInterceptor.WrapInterface(typeof(TInterface));
            ILEmitProxy proxy = new ILEmitProxy(handlers);
            object target = Activator.CreateInstance(typeof(TConcrete));
            List<object> wrappedCtorArgs = new List<object> { proxy, target };
            return (TInterface)Activator.CreateInstance(wrappedType, wrappedCtorArgs.ToArray());
        }

        public interface ISpyOut
        {
            void OutChar(out char outValue);
            void OutComplexValueType(out ComplexValueType outValueType);
            void OutDouble(out double outValue);
            void OutInt16(out short outValue);
            void OutInt32(out int outValue);
            void OutInt64(out long outValue);
            void OutReferenceType(out string outReference);
        }

        sealed class SpyOut : ISpyOut
        {
            public void OutChar(out char outValue)
            {
                outValue = 'a';
            }

            public void OutComplexValueType(out ComplexValueType outValueType)
            {
                outValueType = new ComplexValueType
                                   {
                                       Byte = byte.MaxValue, Char = 'a',
                                       Decimal = decimal.MaxValue,
                                       Double = double.MaxValue,
                                       Float = float.MaxValue,
                                       Int = int.MaxValue,
                                       Long = long.MaxValue,
                                       Short = short.MaxValue,
                                       String = "Hello, world!",
                                       UInt = uint.MaxValue,
                                       ULong = ulong.MaxValue,
                                       UShort = ushort.MaxValue
                                   };
            }

            public void OutDouble(out double outValue)
            {
                outValue = double.MaxValue;
            }

            public void OutInt16(out short outValue)
            {
                outValue = short.MaxValue;
            }

            public void OutInt32(out int outValue)
            {
                outValue = int.MaxValue;
            }

            public void OutInt64(out long outValue)
            {
                outValue = long.MaxValue;
            }

            public void OutReferenceType(out string outReference)
            {
                outReference = "Hello, world!";
            }
        }
    }
}