using System;
using System.Collections.Generic;
using System.Reflection;
using Unity.Extensions;
using Xunit;

namespace Unity.Extensions
{
    public class OutParametersVirtualInterceptorTest
    {
        [Fact]
        public void OutComplexValueType()
        {
            RecordingHandler handler = new RecordingHandler();
            MethodBase method = typeof(SpyOut).GetMethod("OutComplexValueType");
            Dictionary<MethodBase, List<IInterceptionHandler>> dictionary = new Dictionary<MethodBase, List<IInterceptionHandler>>();
            List<IInterceptionHandler> handlers = new List<IInterceptionHandler> { handler };
            dictionary.Add(method, handlers);

            SpyOut result = WrapAndCreateType<SpyOut>(dictionary);
            ComplexValueType outValue;
            result.OutComplexValueType(out outValue);

            Assert.Equal(byte.MaxValue, outValue.Byte);
            Assert.Equal('a', outValue.Char);
            Assert.Equal(decimal.MaxValue, outValue.Decimal);
            Assert.Equal(double.MaxValue, outValue.Double);
            Assert.Equal(float.MaxValue, outValue.Float);
            Assert.Equal(int.MaxValue, outValue.Int);
            Assert.Equal(long.MaxValue, outValue.Long);
            Assert.Equal(short.MaxValue, outValue.Short);
            Assert.Equal("Hello, world!", outValue.String);
            Assert.Equal(uint.MaxValue, outValue.UInt);
            Assert.Equal(ulong.MaxValue, outValue.ULong);
            Assert.Equal(ushort.MaxValue, outValue.UShort);
        }

        [Fact]
        public void OutDouble()
        {
            RecordingHandler handler = new RecordingHandler();
            MethodBase method = typeof(SpyOut).GetMethod("OutDouble");
            Dictionary<MethodBase, List<IInterceptionHandler>> dictionary = new Dictionary<MethodBase, List<IInterceptionHandler>>();
            List<IInterceptionHandler> handlers = new List<IInterceptionHandler> { handler };
            dictionary.Add(method, handlers);

            SpyOut result = WrapAndCreateType<SpyOut>(dictionary);
            double outValue;
            result.OutDouble(out outValue);

            Assert.Equal(double.MaxValue, outValue);
        }

        [Fact]
        public void OutInt16()
        {
            RecordingHandler handler = new RecordingHandler();
            MethodBase method = typeof(SpyOut).GetMethod("OutInt16");
            Dictionary<MethodBase, List<IInterceptionHandler>> dictionary = new Dictionary<MethodBase, List<IInterceptionHandler>>();
            List<IInterceptionHandler> handlers = new List<IInterceptionHandler> { handler };
            dictionary.Add(method, handlers);

            SpyOut result = WrapAndCreateType<SpyOut>(dictionary);
            short outValue;
            result.OutInt16(out outValue);

            Assert.Equal(short.MaxValue, outValue);
        }

        [Fact]
        public void OutInt32()
        {
            RecordingHandler handler = new RecordingHandler();
            MethodBase method = typeof(SpyOut).GetMethod("OutInt32");
            Dictionary<MethodBase, List<IInterceptionHandler>> dictionary = new Dictionary<MethodBase, List<IInterceptionHandler>>();
            List<IInterceptionHandler> handlers = new List<IInterceptionHandler> { handler };
            dictionary.Add(method, handlers);

            SpyOut result = WrapAndCreateType<SpyOut>(dictionary);
            int outValue;
            result.OutInt32(out outValue);

            Assert.Equal(int.MaxValue, outValue);
        }

        [Fact]
        public void OutInt64()
        {
            RecordingHandler handler = new RecordingHandler();
            MethodBase method = typeof(SpyOut).GetMethod("OutInt64");
            Dictionary<MethodBase, List<IInterceptionHandler>> dictionary = new Dictionary<MethodBase, List<IInterceptionHandler>>();
            List<IInterceptionHandler> handlers = new List<IInterceptionHandler> { handler };
            dictionary.Add(method, handlers);

            SpyOut result = WrapAndCreateType<SpyOut>(dictionary);
            long outValue;
            result.OutInt64(out outValue);

            Assert.Equal(long.MaxValue, outValue);
        }

        [Fact]
        public void OutReferenceType()
        {
            RecordingHandler handler = new RecordingHandler();
            MethodBase method = typeof(SpyOut).GetMethod("OutReferenceType");
            Dictionary<MethodBase, List<IInterceptionHandler>> dictionary = new Dictionary<MethodBase, List<IInterceptionHandler>>();
            List<IInterceptionHandler> handlers = new List<IInterceptionHandler> { handler };
            dictionary.Add(method, handlers);

            SpyOut result = WrapAndCreateType<SpyOut>(dictionary);
            string outReference;
            result.OutReferenceType(out outReference);

            Assert.Equal("Hello, world!", outReference);
        }

        static T WrapAndCreateType<T>(IEnumerable<KeyValuePair<MethodBase, List<IInterceptionHandler>>> handlers,
                                      params object[] ctorArgs)
        {
            Type wrappedType = VirtualInterceptor.WrapClass(typeof(T));
            ILEmitProxy proxy = new ILEmitProxy(handlers);
            List<object> wrappedCtorArgs = new List<object> { proxy };
            wrappedCtorArgs.AddRange(ctorArgs);
            return (T)Activator.CreateInstance(wrappedType, wrappedCtorArgs.ToArray());
        }

        public class SpyOut
        {
            public virtual void OutChar(out char outValue)
            {
                outValue = 'a';
            }

            public virtual void OutComplexValueType(out ComplexValueType outValueType)
            {
                outValueType = new ComplexValueType
                                   {
                                       Byte = byte.MaxValue,
                                       Char = 'a',
                                       Decimal = decimal.MaxValue,
                                       Double = double.MaxValue,
                                       Float = float.MaxValue,
                                       Int = int.MaxValue,
                                       Long = long.MaxValue,
                                       Short = short.MaxValue,
                                       String = "Hello, world!",
                                       UInt = uint.MaxValue,
                                       ULong = ulong.MaxValue,
                                       UShort = ushort.MaxValue
                                   };
            }

            public virtual void OutDouble(out double outValue)
            {
                outValue = double.MaxValue;
            }

            public virtual void OutInt16(out short outValue)
            {
                outValue = short.MaxValue;
            }

            public virtual void OutInt32(out int outValue)
            {
                outValue = int.MaxValue;
            }

            public virtual void OutInt64(out long outValue)
            {
                outValue = long.MaxValue;
            }

            public virtual void OutReferenceType(out string outReference)
            {
                outReference = "Hello, world!";
            }
        }
    }
}