using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Microsoft.Practices.Unity.Interception
{
    [TestClass]
    public class InjectionAndInterceptionInterceptionExtensionTest
    {
        [TestMethod]
        public void CanInjectAnInterceptedClass()
        {
            Recorder.Records.Clear();
            IUnityContainer container = new UnityContainer();
            container.AddNewExtension<InterceptionExtension>();
            container.Configure<IInterceptionConfiguration>().InterceptVirtual<SpyVirtual>(typeof(SpyVirtual).GetMethod("InterceptedMethod"),
                                                                                           new RecordingHandler());
            container.RegisterType<IDependencyClass, DependencyClass>();

            IDependencyClass obj = container.Resolve<IDependencyClass>();
            obj.CheckItOut();

            Assert.AreEqual(3, Recorder.Records.Count);
            Assert.AreEqual("Before Method", Recorder.Records[0]);
            Assert.AreEqual("In Method", Recorder.Records[1]);
            Assert.AreEqual("After Method", Recorder.Records[2]);
        }

        public class SpyVirtual
        {
            public virtual void InterceptedMethod()
            {
                Recorder.Records.Add("In Method");
            }
        }

        public interface IDependencyClass
        {
            void CheckItOut();
        }

        public class DependencyClass : IDependencyClass
        {
            readonly SpyVirtual virtualSpy;

            public DependencyClass(SpyVirtual virtualSpy)
            {
                this.virtualSpy = virtualSpy;
            }

            public void CheckItOut()
            {
                virtualSpy.InterceptedMethod();
            }
        }
    }
}