using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Microsoft.Practices.Unity.Interception
{
    [TestClass]
    public class InterceptInterfaceInterceptionExtensionTest
    {
        [TestMethod]
        public void ExceptionsAreUnchanged()
        {
            Recorder.Records.Clear();
            IUnityContainer container = new UnityContainer();
            container.AddNewExtension<InterceptionExtension>();
            container.RegisterType<ISpy, SpyInterface>();
            container.Configure<IInterceptionConfiguration>().InterceptInterface<SpyInterface>(typeof(ISpy).GetMethod("ThrowsException"),
                                                                                               new RecordingHandler());

            ISpy obj = container.Resolve<ISpy>();

            AssertEx.Throws<Exception>(obj.ThrowsException);

            Assert.AreEqual(3, Recorder.Records.Count);
            Assert.AreEqual("Before Method", Recorder.Records[0]);
            Assert.AreEqual("In Method", Recorder.Records[1]);
            Assert.AreEqual("After Method", Recorder.Records[2]);
        }

        [TestMethod]
        public void InterceptViaAttributes()
        {
            Recorder.Records.Clear();
            IUnityContainer container = new UnityContainer();
            container.AddNewExtension<InterceptionExtension>();
            container.RegisterType<ISpy, SpyInterfaceAttributes>();

            ISpy obj = container.Resolve<ISpy>();
            obj.InterceptedMethod();

            Assert.AreEqual(3, Recorder.Records.Count);
            Assert.AreEqual("Before Method", Recorder.Records[0]);
            Assert.AreEqual("In Method", Recorder.Records[1]);
            Assert.AreEqual("After Method", Recorder.Records[2]);
        }

        [TestMethod]
        public void InterceptViaCode()
        {
            Recorder.Records.Clear();
            IUnityContainer container = new UnityContainer();
            container.AddNewExtension<InterceptionExtension>();
            container.RegisterType<ISpy, SpyInterface>();
            container.Configure<IInterceptionConfiguration>().InterceptInterface<SpyInterface>(typeof(ISpy).GetMethod("InterceptedMethod"),
                                                                                               new RecordingHandler());

            ISpy obj = container.Resolve<ISpy>();
            obj.InterceptedMethod();

            Assert.AreEqual(3, Recorder.Records.Count);
            Assert.AreEqual("Before Method", Recorder.Records[0]);
            Assert.AreEqual("In Method", Recorder.Records[1]);
            Assert.AreEqual("After Method", Recorder.Records[2]);
        }

        public interface ISpy
        {
            void InterceptedMethod();
            void ThrowsException();
        }

        internal sealed class SpyInterface : ISpy
        {
            public void InterceptedMethod()
            {
                Recorder.Records.Add("In Method");
            }

            public void ThrowsException()
            {
                Recorder.Records.Add("In Method");
                throw new Exception();
            }
        }

        internal sealed class SpyInterfaceAttributes : ISpy
        {
            [InterfaceIntercept(typeof(RecordingHandler))]
            public void InterceptedMethod()
            {
                Recorder.Records.Add("In Method");
            }

            public void ThrowsException()
            {
                throw new Exception();
            }
        }
    }
}