using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Microsoft.Practices.Unity.Interception
{
    [TestClass]
    public class InterceptVirtualInterceptionExtensionTest
    {
        [TestMethod]
        public void ExceptionsAreUnchanged()
        {
            Recorder.Records.Clear();
            IUnityContainer container = new UnityContainer();
            container.AddNewExtension<InterceptionExtension>();
            container.Configure<IInterceptionConfiguration>().InterceptVirtual<SpyVirtual>(typeof(SpyVirtual).GetMethod("ThrowsException"),
                                                                                           new RecordingHandler());

            SpyVirtual obj = container.Resolve<SpyVirtual>();

            AssertEx.Throws<Exception>(obj.ThrowsException);
            Assert.AreEqual(3, Recorder.Records.Count);
            Assert.AreEqual("Before Method", Recorder.Records[0]);
            Assert.AreEqual("In Method", Recorder.Records[1]);
            Assert.AreEqual("After Method", Recorder.Records[2]);
        }

        [TestMethod]
        public void InterceptViaAttributes()
        {
            Recorder.Records.Clear();
            IUnityContainer container = new UnityContainer();
            container.AddNewExtension<InterceptionExtension>();

            SpyVirtualAttributes obj = container.Resolve<SpyVirtualAttributes>();
            obj.InterceptedMethod();

            Assert.AreEqual(3, Recorder.Records.Count);
            Assert.AreEqual("Before Method", Recorder.Records[0]);
            Assert.AreEqual("In Method", Recorder.Records[1]);
            Assert.AreEqual("After Method", Recorder.Records[2]);
        }

        [TestMethod]
        public void InterceptViaCode()
        {
            Recorder.Records.Clear();
            IUnityContainer container = new UnityContainer();
            container.AddNewExtension<InterceptionExtension>();
            container.Configure<IInterceptionConfiguration>().InterceptVirtual<SpyVirtual>(typeof(SpyVirtual).GetMethod("InterceptedMethod"),
                                                                                           new RecordingHandler());

            SpyVirtual obj = container.Resolve<SpyVirtual>();
            obj.InterceptedMethod();
            obj.NonInterceptedMethod();

            Assert.AreEqual(4, Recorder.Records.Count);
            Assert.AreEqual("Before Method", Recorder.Records[0]);
            Assert.AreEqual("In Method", Recorder.Records[1]);
            Assert.AreEqual("After Method", Recorder.Records[2]);
            Assert.AreEqual("In Non-Intercepted Method", Recorder.Records[3]);
        }

        public class SpyVirtual
        {
            public virtual void InterceptedMethod()
            {
                Recorder.Records.Add("In Method");
            }

            public void NonInterceptedMethod()
            {
                Recorder.Records.Add("In Non-Intercepted Method");
            }

            public virtual void ThrowsException()
            {
                Recorder.Records.Add("In Method");
                throw new Exception("This is my exception!");
            }
        }

        public class SpyVirtualAttributes
        {
            [VirtualIntercept(typeof(RecordingHandler))]
            public virtual void InterceptedMethod()
            {
                Recorder.Records.Add("In Method");
            }
        }
    }
}