﻿using System;
using System.Windows;
using Microsoft.Practices.SLUnity;
using StopLight.Interfaces;
using Stoplight.Views.StatusBar;
using StopLight.Services;
using Stoplight.Interfaces;

namespace Stoplight.Base
{
    public class BootStrapper<TView> where TView : UIElement
    {
        private Application app { get; set; }

        public BootStrapper()
        {
        }

        public void Wireup(Application application)
        {
            app = application;
            app.Startup += new StartupEventHandler(app_Startup);
            app.Exit += new EventHandler(app_Exit);
            app.UnhandledException += new EventHandler<ApplicationUnhandledExceptionEventArgs>(app_UnhandledException);
        }

        private void app_Startup(object sender, StartupEventArgs e)
        {
            IUnityContainer container = new UnityContainer()
                .RegisterType<IEventAggregator, EventAggregator>(new ContainerControlledLifetimeManager())
                .RegisterType<ILoggerFacade, DebugLogger>()
                .RegisterType<IStatusBarView, StatusBarView>()
                .RegisterType<IStoplightService, StoplightService>()
                .RegisterType<IStoplightScheduleService, StoplightScheduleService>()
                .RegisterType<IStoplightTimerService, RealTimeTimerService>();

            container.RegisterInstance<IUnityContainer>(container);

            app.RootVisual = (UIElement)container.Resolve<TView>();
        }

        private void app_Exit(object sender, EventArgs e)
        {

        }
        
        void app_UnhandledException(object sender, ApplicationUnhandledExceptionEventArgs e)
        {
            // If the app is running outside of the debugger then report the exception using
            // the browser's exception mechanism. On IE this will display it a yellow alert 
            // icon in the status bar and Firefox will display a script error.
            if (!System.Diagnostics.Debugger.IsAttached)
            {

                // NOTE: This will allow the application to continue running after an exception has been thrown
                // but not handled. 
                // For production applications this error handling should be replaced with something that will 
                // report the error to the website and stop the application.
                e.Handled = true;

                try
                {
                    string errorMsg = e.ExceptionObject.Message + e.ExceptionObject.StackTrace;
                    errorMsg = errorMsg.Replace('"', '\'').Replace("\r\n", @"\n");

                    System.Windows.Browser.HtmlPage.Window.Eval("throw new Error(\"Unhandled Error in Silverlight 2 Application " + errorMsg + "\");");
                }
                catch (Exception)
                {
                }
            }
        }
    }
}
