using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Configuration;
using Common.Unity.Configuration;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Microsoft.Practices.ObjectBuilder2;
using Microsoft.Practices.Unity;
using Microsoft.Practices.Unity.Configuration;

namespace Common.Unity.Extensions.Configuration
{
	public class ConfigurationExtensionConfigurator : IContainerConfigurationCommand
	{
		public void Configure( IUnityContainer container )
		{
			var configuration = container.Configure<IConfigurationExtensionConfiguration>();
			if ( configuration != null )
			{
				// Add any additional build strategies:
				using ( var child = container.CreateChildContainer() )
				{
					foreach ( var strategy in Strategies )
					{
						configuration.AddStrategy( (IBuilderStrategy)child.Resolve( strategy.Type ), strategy.Stage );
					}
				}

				// Set any default policies:
				foreach ( var element in DefaultPolicies )
				{
					configuration.AddDefaultPolicy( element.BuildType, element.CreatePolicy( container ) );
				}
			}
		}

		public ObservableCollection<PolicyReference> DefaultPolicies
		{
			get { return defaultPolicies ?? ( defaultPolicies = new ObservableCollection<PolicyReference>() ); }
		}	ObservableCollection<PolicyReference> defaultPolicies;

		public ObservableCollection<StrategyReference> Strategies
		{
			get { return strategies ?? ( strategies = new ObservableCollection<StrategyReference>() ); }
		}	ObservableCollection<StrategyReference> strategies;
	}
}