using Common.Unity.Testing.Objects;
using Microsoft.Practices.Unity;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Testing.Core;

namespace Common.Unity.Testing
{
	[TestClass]
	public class ConfigurationTests : TestBase
	{
		[TestMethod, Owner( TestOwners.MichaelD_Bang )]
		public void VerifyConfigurationCreatesContainerCorrectly()
		{
			var container = Configuration.Container;
			Assert.IsNotNull( container );
		}

		[TestMethod, Owner( TestOwners.MichaelD_Bang )]
		public void VerifyTypesRegisteredCorrectly()
		{
			var item = Configuration.Container.Resolve<INamedObject>();
			Assert.IsNotNull( item );
			Assert.IsInstanceOfType( item, typeof(NamedObject) );
		}

		[TestMethod, Owner( TestOwners.MichaelD_Bang )]
		public void VerifyInjectionPropertyRegisteredCorrectly()
		{
			var item = Configuration.Container.Resolve<INamedObject>();
			Assert.AreEqual( "Mapped Named Object", item.Name );
		}

		[TestMethod, Owner( TestOwners.MichaelD_Bang )]
		public void VerifyInjectionPropertyRegisteredCorrectlyWithType()
		{
			var item = Configuration.Container.Resolve<INamedTypeObject>( "Default" );
			Assert.AreEqual( "Mapped Named-Type Object", item.Name );
			Assert.AreEqual( typeof(string), item.Type );
		}

		[TestMethod, Owner( TestOwners.MichaelD_Bang )]
		public void VerifyInjectionPropertyRegisteredCorrectlyWithIntType()
		{
			var item = Configuration.Container.Resolve<INamedTypeObject>( "Normal" );
			Assert.AreEqual( "Named-Type Object", item.Name );
			Assert.AreEqual( typeof(int), item.Type );
		}

		[TestMethod, Owner( TestOwners.MichaelD_Bang )]
		public void VerifyInjectionPropertyForGenericPropertyRegisteredCorrectly()
		{
			var item = Configuration.Container.Resolve<GenericObject<INamedTypeObject>>();
			Assert.AreEqual( "Named-Type Object", item.Property.Name );
		}

		[TestMethod, Owner( TestOwners.MichaelD_Bang )]
		public void VerifyConstructorInjection()
		{
			var item = Configuration.Container.Resolve<ConstructorObject>();
			Assert.AreEqual( "String-based Construction!", item.Message );
		}

		[TestMethod, Owner( TestOwners.MichaelD_Bang )]
		public void VerifyConstructorInjectionWithInt()
		{
			var item = Configuration.Container.Resolve<ConstructorObject>( "Int" );
			Assert.AreEqual( 6776, item.Number );
		}

		[TestMethod, Owner( TestOwners.MichaelD_Bang )]
		public void VerifyMethodInjection()
		{
			var item = Configuration.Container.Resolve<MethodObject>();
			Assert.AreEqual( "Hello Method Call!", item.Message );
		}
	}
}