using System.Globalization;
using Microsoft.Practices.ObjectBuilder2;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Testing.Core;

namespace Common.Unity.Testing
{
	[TestClass]
	public class NamedTypeBuildKeyConverterTests : TestBase
	{
		[TestMethod, Owner( TestOwners.MichaelD_Bang )]
		public void VerifyNamedTypeBuildKeyConverterConvertsToKeyCorrectly()
		{
			string name = "testing";
			NamedTypeBuildKeyConverter converter = new NamedTypeBuildKeyConverter();
			string input = new NamedTypeBuildKey( typeof(string), name ).ToString();
			NamedTypeBuildKey result = (NamedTypeBuildKey)converter.ConvertFrom( null, CultureInfo.CurrentCulture, input );
			Assert.AreEqual( typeof(string), result.Type );
			Assert.AreEqual( name, result.Name );
		}

		[TestMethod, Owner( TestOwners.MichaelD_Bang )]
		public void VerifyNamedTypeBuildKeyConverterConvertsFullyQualifiedTypeToKeyCorrectly()
		{
			string name = "testing";
			NamedTypeBuildKeyConverter converter = new NamedTypeBuildKeyConverter();
			string input = string.Format( "Build Key[{0}, {1}]", typeof(NamedTypeBuildKeyConverter).AssemblyQualifiedName, name );
			NamedTypeBuildKey result = (NamedTypeBuildKey)converter.ConvertFrom( null, CultureInfo.CurrentCulture, input );
			Assert.AreEqual( typeof(NamedTypeBuildKeyConverter), result.Type );
			Assert.AreEqual( name, result.Name );
		}

		[TestMethod, Owner( TestOwners.MichaelD_Bang )]
		public void VerifyNamedTypeBuildKeyConverterConvertsToKeyWithNullNameCorrectly()
		{
			NamedTypeBuildKeyConverter converter = new NamedTypeBuildKeyConverter();
			string input = new NamedTypeBuildKey( typeof(string), null ).ToString();
			NamedTypeBuildKey result = (NamedTypeBuildKey)converter.ConvertFrom( null, CultureInfo.CurrentCulture, input );
			Assert.AreEqual( typeof(string), result.Type );
			Assert.IsNull( result.Name );
		}

	}
}